(* ==================================================================
The use of this Package is described in "Economic and Financial
Modeling with Mathematica", Hal Varian, editor, TELOS/Springer-Verlag,
1993, ISBN 0-387-97882-8.  $44.95 (Includes 1.44 MS-DOS disk.)  To
place order call 1-800-777-4643 or fax 1-201-348-4505.
=====================================================================*)

BeginPackage["ktpackage`"]

DMax::usage="DMax[funct,var] takes as input a list
of functions funct and a list of variables vars, 
generates from it a set of equations eqns and then
numerically solves the system.  Thus it solve the
first order conditions."

LangMax::usage="LangMax[funct,const,var] takes as 
input a list of functions funct, a list of 
constraints const and a list of variables vars.  
It then generates a Lagrangean for the system, 
sets its partial derivatives with respect to 
the variables equal to zero and then solves 
the system.  Thus it solves the a constrained 
optimization problem."

KTMax::usage="KTMax[funct,const,var] takes as 
input a list of functions funct, a list of 
constraints const and a list of variables vars.  
It assumes a particular set of constraints is 
binding. It then generates a Lagrangean for 
this system, sets its partial derivatives with 
respect to the variables equal to zero and then 
solves the system.  It stores this solution if 
it is feasible with respect to the remaining 
constraints.  Thus it solves a constrained 
optimization problem assuming a particular set of 
constraints is binding andchecks if that is true. 
It then works through all possible sets of binding 
constraints to identify the feasible solution that 
mazimizes the value of the objective function."

Begin["`Private`"]

DMax[funct_,vars_]:=Block[{t},
  eqns=Table[D[funct,vars[[t]] ]==0,
  {t,1,Length[vars]}];
  NSolve[eqns,vars] ];

LangMax[funct_,const_,vars_]:=Block[{t,lagran,varlag},
   varlag=Table[lam[t],{t,1,Length[const]}];
   lagran=funct+varlag.const;
   DMax[lagran,Join[vars,varlag]] ];
   
KTMax[funct_,const_,vars_]:=
Block[{t,xxx,yyy,t2,t3,l,i,f,j,anslist,thelist,
vlist,lagran,varlag,try,plist},
 l=Length[const];
 thelist=Table[Mod[Floor[j/2^i],2],
 	{j,1,2^l-1},{i,0,l-1}];   
 anslist={};
    For[t=1,t<=Length[thelist],t++,
      try=thelist[[t]];
      varlag=Table[lam[t2]*try[[t2]],
      	{t2,1,Length[const]}];
      vlist=Select[varlag,Function[x,
      Not[NumberQ[x]]]];
      lagran=funct+varlag.const;     
      anslist=Join[anslist,
      	DMax[lagran,Join[vars,vlist]]];
    ];
    f=Function[xxx,Apply[And,Map[Function[yyy,
    	NonNegative[Chop[yyy]] && 
    	NumberQ[yyy]],const/.xxx]]];
    anslist=Select[anslist,f];
    Maxlist=Map[Function[xxx,funct/.xxx],anslist];
    listmax=Max[Maxlist];
    plist=Position[Maxlist,listmax];
    Clear[t];
    anslist=Table[anslist[[plist[[t]]]],
    	{t,1,Length[plist]}];
    varlag=Table[lam[t],{t,1,Length[const]}];
    vlist=Join[vars,varlag];
f=Function[xxx,Table[vlist[[t2]]-> 
	(vlist[[t2]]/.xxx/.Table[lam[t]->0
  ,{t,1,Length[const]}])[[1]],{t2,1,Length[vlist]}]];
    anslist=Map[f,anslist];
    Return[{listmax,anslist}/.lam->"lam"];
 ];
 
 End[]
 EndPackage[]




